@@ -38,8 +38,8 @@ import time
 import zlib
 
 import talisker
-import talisker.request_id
 import talisker.logs
+import talisker.requests
 from talisker.util import (
     get_rounded_ms,
     module_cache,
@@ -54,7 +54,8 @@ __all__ = [
     'configure_testing',
     'get_client',
     'record_breadcrumb',
-    'report_wsgi_error',
+    'report_wsgi',
+    'new_context',
     'set_client',
 ]
 
@@ -74,15 +75,19 @@ try:
     import raven.middleware
     import raven.breadcrumbs
     import raven.handlers.logging
+    from raven.transport.requests import RequestsHTTPTransport
 except ImportError:
     # dummy APIs that do nothing
+    def new_context():
+        pass
+
     def record_breadcrumb(*args, **kwargs):
         pass
 
     def record_log_breadcrumb(record):
         pass
 
-    def report_wsgi_error(environ, msg=None, **kwargs):
+    def report_wsgi(http_context, exc_info, msg=None, **kwargs):
         return
 
     class TestSentryContext():
@@ -113,6 +118,14 @@ else:
     raven.breadcrumbs.ignore_logger('talisker.slowqueries')
     raven.breadcrumbs.ignore_logger('talisker.requests')
 
+    FILTERED_EXTRAS = {
+        'request_id',  # do not need request_id for each log breadcrumb
+        'exc_type',    # exc info already displayed
+    }
+
+    def new_context():
+        get_client().context.activate()
+
     def record_breadcrumb(*args, **kwargs):
         raven.breadcrumbs.record(*args, **kwargs)
 
@@ -125,7 +138,7 @@ else:
             record.message,
             record.args,
             {
-                'extra': getattr(record, '_structured', {}),
+                'extra': getattr(record, 'extra', {}),
                 'exc_info': record.exc_info,
                 'stack_info': getattr(record, 'stack_info', None)
             },
@@ -142,30 +155,53 @@ else:
 
         def processor(data):
             metadata = {
-                'path': record.pathname,
-                'lineno': record.lineno,
+                'location': '{}:{}:{}'.format(
+                    record.pathname,
+                    record.lineno,
+                    getattr(record, 'funcName', ''),
+                ),
             }
-            if hasattr(record, 'func'):
-                metadata['func'] = record.func
-            metadata.update(getattr(record, '_structured', {}))
+            extra = getattr(record, 'extra', {})
+            metadata.update(
+                (k, str(v)) for k, v in extra.items()
+                if k not in FILTERED_EXTRAS
+            )
             data.update({
-                'message': record.message,
+                'message': record.getMessage(),
                 'category': record.name,
                 'level': record.levelname.lower(),
                 'data': metadata,
             })
         raven.breadcrumbs.record(processor=processor)
 
-    def report_wsgi_error(environ, msg=None, **kwargs):
-        """Use raven to report error"""
+    def report_wsgi(http_context, exc_info=None, msg='wsgi request', **kwargs):
+        """Resport a wsgi request to raven"""
         sentry = get_client()
-        # reuse code from Sentry middleware, if a bit unpleasently
-        mw = raven.middleware.Sentry(None, sentry)
-        sentry.http_context(mw.get_http_context(environ))
-        if msg is None:
-            return sentry.captureException(**kwargs)
-        else:
+        sentry.http_context(http_context)
+        if exc_info is None:
             return sentry.captureMessage(msg, **kwargs)
+        else:
+            return sentry.captureException(exc_info, **kwargs)
+
+    class TaliskerRequestsTransport(RequestsHTTPTransport):
+        """Requests transport that uses Talisker's persistant requests session.
+
+        This provides connection pooling, logging and metrics.
+        """
+        scheme = ['talisker+http', 'talisker+https']
+
+        def send(self, url, data, headers):
+            if self.verify_ssl:
+                # If SSL verification is enabled use the provided CA bundle to
+                # perform the verification.
+                self.verify_ssl = self.ca_certs
+            talisker.requests.get_session().post(
+                url,
+                data=data,
+                headers=headers,
+                verify=self.verify_ssl,
+                timeout=self.timeout,
+            )
 
     class TaliskerSentryClient(raven.Client):
 
@@ -226,6 +262,7 @@ else:
 
         def start(self):
             self.client = get_client()
+            new_context()
             self.orig_remote = self.client.remote
             self.client.set_dsn(self.dsn, DummySentryTransport)
             self.transport = self.client.remote.get_transport()
@@ -287,6 +324,9 @@ def ensure_talisker_config(kwargs):
     processors = set(kwargs.get('processors') or [])
     kwargs['processors'] = list(default_processors | processors)
 
+    if 'transport' not in kwargs:
+        kwargs['transport'] = TaliskerRequestsTransport
+
     # note: style clash - sentry client api is 'sanitize_keys'
     sanitise_keys = kwargs.get('sanitize_keys', [])
     if sanitise_keys is None:  # flask integration explicitly sets None
@@ -356,7 +396,7 @@ def log_client(client):
 
 
 def add_talisker_context(data):
-    rid = talisker.request_id.get()
+    rid = talisker.Context.request_id
     if rid:
         data['tags']['request_id'] = rid
     data['extra'].update(talisker.logs.logging_context.flat)
@@ -393,7 +433,7 @@ def add_talisker_context(data):
 def sql_summary(sql_crumbs, start_time):
 
     def duration(crumb):
-        return float(crumb['data'].get('duration', 0))
+        return float(crumb['data'].get('duration_ms', 0))
 
     sql_crumbs.sort(key=duration, reverse=True)
     sql_time = sum(duration(c) for c in sql_crumbs)
@@ -406,13 +446,14 @@ def sql_summary(sql_crumbs, start_time):
         sql_summary['non_sql_time'] = request_time - sql_time
         sql_summary['total_time'] = request_time
 
-    sql_summary['slowest queries'] = [
-        OrderedDict([
-            ('duration', c['data']['duration']),
+    slowest = []
+    for c in sql_crumbs[:5]:
+        slow = OrderedDict([
+            ('duration_ms', c['data']['duration_ms']),
             ('query', c['data']['query'])]
         )
-        for c in sql_crumbs[:5]
-    ]
+        slowest.append(slow)
+    sql_summary['slowest queries'] = slowest
 
     return sql_summary
 
