@@ -33,13 +33,15 @@ __metaclass__ = type
 import errno
 import functools
 import logging
+import os
 import pkg_resources
 import sys
+import threading
 import time
 
 import werkzeug.local
 from future.moves.urllib.parse import urlparse
-from future.utils import text_to_native_str
+import future.utils
 
 
 # look up table for errno's
@@ -59,6 +61,32 @@ ERROR_CODES[-11] = 'EAI_SYSTEM'
 ERROR_CODES[-12] = 'EAI_OVERFLOW'
 
 
+EARLY_LOGS = []
+EARLY_LOGS_PROCESSED = False
+
+
+def early_log(name, level, *args, **kwargs):
+    """Logger wrap for talisker startup code.
+
+    Collects logs for later processing when logging is initialised
+    """
+    if EARLY_LOGS_PROCESSED:
+        logger = logging.getLogger(name)
+        getattr(logger, level)(*args, **kwargs)
+    else:
+        EARLY_LOGS.append((name, level, args, kwargs))
+
+
+def flush_early_logs():
+    global EARLY_LOGS, EARLY_LOGS_PROCESSED
+    EARLY_LOGS_PROCESSED = True
+
+    # process pending logs
+    for name, level, args, kwargs in EARLY_LOGS:
+        early_log(name, level, *args, **kwargs)
+    EARLY_LOGS[:] = []
+
+
 def parse_url(url, proto='http'):
     # urlparse won't parse properly without a protocol
     if '://' not in url:
@@ -93,13 +121,13 @@ def force_unicode(s):
 
 def set_wsgi_header(headers, name, value):
     """Replace a wsgi header, ensuring correct encoding"""
-    native_name = text_to_native_str(name)
+    native_name = future.utils.text_to_native_str(name)
     for i, (k, v) in enumerate(headers):
         if native_name == k:
-            headers[i] = (native_name, text_to_native_str(value))
+            headers[i] = (native_name, future.utils.text_to_native_str(value))
             return
 
-    headers.append((native_name, text_to_native_str(value)))
+    headers.append((native_name, future.utils.text_to_native_str(value)))
 
 
 def get_rounded_ms(start_time, now_time=None):
@@ -196,12 +224,6 @@ def clear_globals():
         d.clear()
 
 
-def context_local():
-    local = werkzeug.local.Local()
-    _context_locals.append(local)
-    return local
-
-
 def clear_context_locals():
     for local in _context_locals:
         werkzeug.local.release_local(local)
@@ -236,3 +258,36 @@ def get_errno_fields(exc):
     if getattr(root, 'filename2', None) is not None:
         fields['filename2'] = root.filename2
     return fields
+
+
+if future.utils.PY3:
+    def datetime_to_timestamp(dt):
+        return dt.timestamp()
+else:
+    def datetime_to_timestamp(dt):
+        time.mktime(dt.utctimetuple())
+
+
+class Local(object):
+    """Wrap a threading.local that will be cleared on fork."""
+    def __init__(self):
+        self._local = threading.local()
+        self._local._pid = os.getpid()
+
+    def _check(self):
+        pid = os.getpid()
+        if self._local._pid != pid:
+            self._local = threading.local()
+            self._local._pid = pid
+
+    # proxy through to _local
+    def __getattr__(self, item):
+        self._check()
+        return getattr(self._local, item)
+
+    def __setattr__(self, item, value):
+        if item == '_local':
+            super(Local, self).__setattr__(item, value)
+        else:
+            self._check()
+            setattr(self._local, item, value)
